package com.emsoft.grails

import com.emsoft.common.Assert
import com.emsoft.common.StringHelper
import com.emsoft.exceptions.ExceptionHelper
import com.emsoft.io.IOHelper
import grails.validation.ValidationException
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import org.codehaus.groovy.grails.commons.ApplicationHolder
import org.codehaus.groovy.grails.context.support.PluginAwareResourceBundleMessageSource
import org.codehaus.groovy.grails.web.context.ServletContextHolder
import org.codehaus.groovy.grails.web.pages.GroovyPagesTemplateEngine
import org.codehaus.groovy.grails.web.servlet.mvc.GrailsHttpSession
import org.codehaus.groovy.grails.web.servlet.mvc.GrailsParameterMap
import org.codehaus.groovy.grails.web.servlet.mvc.GrailsWebRequest
import org.springframework.web.context.request.RequestContextHolder

class GrailsHelper {

    static void checkErrors(command) {
        if (command.hasErrors())
            ExceptionHelper.logAndThrow(new ValidationException(null, command.errors))
    }

    static String getErrors(instance, delimeter = ", ") {
        def messageSource = ApplicationHolder.application.mainContext.messageSource as PluginAwareResourceBundleMessageSource
        instance.errors.fieldErrors.collect { messageSource.getMessage(it, null) }.join(delimeter)
    }

    static GrailsWebRequest getWebRequest() {
        RequestContextHolder.currentRequestAttributes() as GrailsWebRequest
    }

    static HttpServletRequest getRequest() {
        webRequest.currentRequest
    }

    static HttpServletResponse getResponse() {
        webRequest.currentResponse
    }

    static GrailsParameterMap getParams() {
        webRequest.params
    }

    static boolean isAjaxRequest() {
        request.xhr
    }

    static GrailsHttpSession getSession() {
        webRequest.session
    }

    static <T> T getBean(Class<T> type) {
        Assert.notNull(type, "type")
        ApplicationHolder.application.mainContext.getBean(type)
    }

    static <T> T getBean(String name) {
        Assert.hasText(name, "name")
        ApplicationHolder.application.mainContext.getBean(name) as T
    }

    static <T> T getBeanByName(Class<T> type) {
        Assert.notNull(type, "type")
        getBean(StringHelper.toFirstLowerCase(type.simpleName))
    }

    static String getContextPath() {
        ServletContextHolder.servletContext.contextPath
    }

    static String getRealPath(String path) {
        ServletContextHolder.servletContext.getRealPath(path)
    }

    /**
     * @param fileContextPath context path to file include file name
     * @param fileData
     * @param rewrite
     * @return
     */
    static File createFile(String fileContextPath, Closure fileData, boolean rewrite = false) {
        Assert.hasText(fileContextPath, "fileContextPath")
        IOHelper.getFile(getRealPath(fileContextPath), rewrite, fileData)
    }

    static GroovyPagesTemplateEngine getTemplateEngine() {
        getBean(GroovyPagesTemplateEngine)
    }

    /**
     * пример использования: render(template: GrailsHelper.createTemplate(page))
     * @param contextDirectory context path without file name
     * @param templateName name of template to use in controller without context path
     * @param markup gsp code of template to render
     * @param rewrite
     * @return
     */
    static String createTemplate(String contextDirectory, String templateName,
            Closure markup, boolean rewrite = false) {
        Assert.hasText(contextDirectory, "contextDirectory")
        Assert.hasText(templateName, "templateName")
        createFile(IOHelper.joinPath(contextDirectory, "_${templateName}.gsp".toString()), markup, rewrite)
        IOHelper.joinPath(true, contextDirectory, templateName)
    }

    /**
     * @param markup gsp code of template
     * @return
     */
    static Writable createTemplate(String markup) {
        templateEngine.createTemplate(markup, request.requestURI).make()
    }

    /**
     * Dynamic compile gsp code to HTML.
     * @param markup
     * @return HTML code for client.
     */
    static String getHTML(String markup) {
        createTemplate(markup).writeTo(new StringWriter()).toString()
    }
}
